local _Questic = _Questic_ADDON_Globals;
local Pool = CreatePoolCollection();
local MinTimeBetweenUpdates = 1;
local LastUpdateTime = 0;

local ClickToShare = "Left-Click to share quest";
local CollapsedHeaders = {};

local function NotEmptyHeaderCheck(QuestIndex)
	if QuestDisplayChecker(QuestIndex + 1) and select(4,GetQuestLogTitle(QuestIndex)) and not select(4,GetQuestLogTitle(QuestIndex + 1)) then --filter out empty headers
		return true;
	end
	return false;
end 

local CurrentlySelected = nil;
local function ShareQuest_OnClick(self,button,userInput)
	if self.QuestID then 
		if self.RequestTarget then 
			_Questic.DataDistribution("REQUEST",self.RequestTarget,self.QuestID)
		else 
			_Questic.ShareQuest(nil,self.QuestID);
		end 
	end 
end 

local RefreshScript = CreateFrame("FRAME");
local function SetAutoRefreshScript(self)
	RefreshScript.self = self; 
	if _Questic.GetVariable("QuestCompareAutoRefresh") then 
		RefreshScript.self.TimerValue = 0;
		RefreshScript:SetScript("OnUpdate",function(self,elapsed)
			self.self.TimerValue = self.self.TimerValue + elapsed;
			if _Questic.GetVariable("CompareFrequency") <= self.self.TimerValue then
				Questic_ADDON_GroupTrack:Refresh();
				self.self.TimerValue = 0;
			end 
			self.self.RefreshButton:SetText("Refresh (in "..floor(_Questic.GetVariable("CompareFrequency") - self.self.TimerValue).."s)");
		end)
	else
		RefreshScript:SetScript("OnUpdate",nil);
		RefreshScript.self.RefreshButton:SetText("Refresh");
	end 
end

local function QuestHeader_OnClick(self,button,userInput)
	if type(self.QuestID) == "string" then 
		local Name = self.QuestID; 
		if button == "RightButton" then 
			CollapsedHeaders[Name] = nil;
		else 
			if type(CollapsedHeaders[Name]) == "boolean" then 
				CollapsedHeaders[Name] = not CollapsedHeaders[Name]; 
			else 
				CollapsedHeaders[Name] = not self.Collapsed;
			end
		end 
		Questic_ADDON_GroupTrack:Refresh(true);		
	end 
end

local GlobalQuestLog = {};
local function PartyQuestLogSync()
	--player quest log
	GlobalQuestLog = {};
	local PlayerQuestlog = _Questic.GetQuestLogTable(); 
	for QuestID,QuestInfo in pairs(PlayerQuestlog) do 
		if QuestInfo.Type == "H" then 
			GlobalQuestLog[QuestID] = GlobalQuestLog[QuestID] or {Count = 0}; 
		elseif QuestInfo.Type == "NQ" then 
			GlobalQuestLog[QuestInfo.HeaderID] = GlobalQuestLog[QuestInfo.HeaderID] or {Count = 0}; 
			GlobalQuestLog[QuestInfo.HeaderID].Count = GlobalQuestLog[QuestInfo.HeaderID].Count + 1;
			GlobalQuestLog[QuestInfo.HeaderID][QuestID] = GlobalQuestLog[QuestInfo.HeaderID][QuestID] or {Name = QuestInfo.Name, Pushable = QuestInfo.Pushable, MembersOnQuest  = {}, Status = ""};
		end 
	end 
	--party quest log 
	local RemoveNames = {};
	for Name,QuestLog in pairs(_Questic.PartyQuestLog) do 
		if QuestLog then 
			if not UnitInParty(Name) or Name == UnitName("player") then 
				table.insert(RemoveNames,Name);
			else 
				for QuestID,QuestInfo in pairs(QuestLog) do 
					if QuestInfo.Type == "H" then 
						GlobalQuestLog[QuestID] = GlobalQuestLog[QuestID] or {Count = 0}; 
					elseif QuestInfo.Type == "NQ" then  
						local Status = "";
						if IsQuestFlaggedCompleted(QuestID) then 
							Status = "C";
						elseif not GetQuestLogIndexByID(QuestID) or GetQuestLogIndexByID(QuestID) == 0 then 
							Status = "M";
						end 
						GlobalQuestLog[QuestInfo.HeaderID] = GlobalQuestLog[QuestInfo.HeaderID] or {Count = 0}; 
						GlobalQuestLog[QuestInfo.HeaderID].Count = GlobalQuestLog[QuestInfo.HeaderID].Count + 1;
						GlobalQuestLog[QuestInfo.HeaderID][QuestID] = GlobalQuestLog[QuestInfo.HeaderID][QuestID] or {Name = QuestInfo.Name, Pushable = QuestInfo.Pushable, MembersOnQuest  = {}, Status = ""};
						GlobalQuestLog[QuestInfo.HeaderID][QuestID].Status = Status;
						GlobalQuestLog[QuestInfo.HeaderID][QuestID].MembersOnQuest[Name] = true;	
					end 
				end 
			end 
		end 
	end 
	for i,v in ipairs(RemoveNames) do 
		_Questic.PartyQuestLog[v] = nil; 
	end 
	return true; 
end 

--select(8,GetQuestLogTitle(QuestlosINFex))
--Questic_ADDON_GroupTrack:QuestCompareRefreshFunc
local function SetAnchor(self,CurrentFrame,LastFrame,QuestID)
	if LastFrame then 
		self[CurrentFrame]:SetPoint("TOPLEFT",self[LastFrame],"BOTTOMLEFT");
	else 
		self[CurrentFrame]:SetPoint("TOPLEFT",self,"TOPLEFT",5,-self.Header:GetHeight()/2);
	end 
	self[CurrentFrame]:SetWidth(self:GetWidth() - 10);
	self[CurrentFrame].QuestID = QuestID;
	self[CurrentFrame]:Show()
end 

local function GroupTrackRefresh(self,ForceRefresh)
	if abs(LastUpdateTime - GetTime()) < MinTimeBetweenUpdates and not ForceRefresh then 
		return nil;
	end 
	local Change = self:PartyQuestLogSync();
	if not Change and not ForceRefresh then 
		return nil;
	end 
	
	local self = Questic_ADDON_GroupTrack;
	
	if self then 
		self:SetScript("OnUpdate",nil);
		if not _Questic.GetVariable("Enable") or not _Questic.GetVariable("GroupTrack") then 
			Questic_ADDON_GroupTrack.ActivePreview = false; 
			return self:Hide(); 
		elseif not (_Questic.GetVariable("QuestCompareInGroup") and IsInGroup() and not IsInRaid()) and not (_Questic.GetVariable("QuestCompareInRaid") and IsInRaid()) and not Questic_ADDON_GroupTrack.ActivePreview then 
			return self:Hide()
		end 
		Pool:ReleaseAll();
		self.RefreshButton:Disable();
		self.TimerValue = 0;
		self.RefreshButton:SetText("Refrash (Processing)");
		--Checking group members
		local UnitsToCheck = {};
		if IsInGroup() then 
			for i = 1,4 do 
				if UnitName("PARTY"..i) then 
					table.insert(UnitsToCheck,{ID = "PARTY"..i,Name = UnitName("PARTY"..i)});
				end 
			end 
		elseif IsInRaid() then 
			for i = 1,40 do 
				if UnitName("RAID"..i) and UnitGUID("RAID"..i) ~= UnitGUID("player") then 
					table.insert(UnitsToCheck,{ID = "RAID"..i,Name = UnitName("RAID"..i)});
				end 
			end 			
		end 
		--Running quest checker
		if getn(UnitsToCheck) >= 0 then 
			local TotalQuests = 0;
			local Collapsed = false;
			local TotalHeaders = 0;
			local LastFrame;
			for HeaderID,HeaderInfo in pairs(GlobalQuestLog) do 
				if HeaderID and HeaderID ~= "" and HeaderInfo.Count and HeaderInfo.Count > 0 then 
					TotalHeaders = TotalHeaders + 1;
					local ClickToExpand = "";
					local HeaderTooltip = "Left-Click: Disable auto expand/collapse and ";
					if type(CollapsedHeaders[HeaderID]) == "boolean" then 
						Collapsed = CollapsedHeaders[HeaderID];
						ClickToExpand = "|TInterface\\PETBATTLES\\PetBattle-LockIcon:0|t";
					else 
						if GetRealZoneText() == HeaderID then 
							Collapsed = false; 
						else 
							Collapsed = true; 
						end 
					end 
					if Collapsed then 
						--ClickToExpand = "[Click to Expand]";
						HeaderTooltip = HeaderTooltip.."expand";
					else 
						--ClickToExpand = "[Click to Collapse]";
						HeaderTooltip = HeaderTooltip.."collapse";
					end 
					HeaderTooltip = HeaderTooltip.." the header.\n\n Right-Click to re-enable auto expand/collapse.";
					self["QuestHeader"..TotalHeaders] = Pool:Acquire("Questic_ADDON_GroupTrackQuestHeaderTemplate");
					self["QuestHeader"..TotalHeaders].LText:SetText(HeaderID);
					self["QuestHeader"..TotalHeaders].RText:SetText(ClickToExpand);
					self["QuestHeader"..TotalHeaders]:SetScript("OnClick",QuestHeader_OnClick);
					self["QuestHeader"..TotalHeaders].Collapsed = Collapsed;
					self["QuestHeader"..TotalHeaders].Tooltip = HeaderTooltip;
					SetAnchor(self,"QuestHeader"..TotalHeaders,LastFrame,HeaderID)
					LastFrame = "QuestHeader"..TotalHeaders;
					if not Collapsed then  
						for QuestID,QuestInfo in pairs(HeaderInfo) do 
							if type(QuestID) == "number" then 
								TotalQuests = TotalQuests + 1;
								self["QuestFrame"..TotalQuests] = Pool:Acquire("Questic_ADDON_GroupTrackQuestTemplate");
								local OnQuest = 0;
								local ToolTipString = QuestInfo.Name
								local PrefixString = "";
								if QuestInfo.Status == "M" then 
									PrefixString = "[|cFFFF0000M|r] ";
									ToolTipString = ToolTipString.."\n|cFFFF0000Quest missing|r";
								elseif QuestInfo.Status == "C" then 
									PrefixString = "[|cFF00FF00C|r] ";
									ToolTipString = ToolTipString.."\n|cFF00FF00Quest completed|r";
								end 
								ToolTipString = ToolTipString.."\n\n|cFFFFFFFFPlayers on quest: \n";
								if QuestInfo.Status == "M" then 
									ToolTipString = ToolTipString.."|ON_QUEST| \n\nPlayers not on quest: \n"..UnitName("player")..", |NOT_ON_QUEST||r";
								elseif QuestInfo.Status == "" then 
									ToolTipString = ToolTipString..UnitName("player")..", |ON_QUEST| \n\nPlayers not on quest: \n|NOT_ON_QUEST||r";
								else 
									ToolTipString = ToolTipString.."|ON_QUEST| \n\nPlayers not on quest: \n|NOT_ON_QUEST||r";
								end 								
								for _,Unit in ipairs(UnitsToCheck) do 
									local NameToCheck = UnitFullName(Unit.ID)
									if select(2,UnitFullName(Unit.ID)) and select(2,UnitFullName(Unit.ID)) ~= "" then 
										NameToCheck = NameToCheck.."-"..select(2,UnitFullName(Unit.ID));
									end  
									if QuestInfo.MembersOnQuest[NameToCheck] then 
										OnQuest = OnQuest + 1;
										if QuestInfo.Pushable == "1" and QuestInfo.Status == "M" then 
											self["QuestFrame"..TotalQuests].RequestTarget = UnitName(Unit.ID);
										else 
											self["QuestFrame"..TotalQuests].RequestTarget = nil;
										end 
										ToolTipString = ToolTipString:gsub("|ON_QUEST|",Unit.Name..", |ON_QUEST|");
									elseif IsUnitOnQuestByQuestID(QuestID,Unit.ID) then 
										OnQuest = OnQuest + 1;
										ToolTipString = ToolTipString:gsub("|ON_QUEST|",Unit.Name..", |ON_QUEST|");
									else
										ToolTipString = ToolTipString:gsub("|NOT_ON_QUEST|",Unit.Name..", |NOT_ON_QUEST|");
									end 
								end 		
								ToolTipString = ToolTipString:gsub(", |ON_QUEST|","");
								ToolTipString = ToolTipString:gsub("|ON_QUEST|","");
								ToolTipString = ToolTipString:gsub(", |NOT_ON_QUEST|","");
								ToolTipString = ToolTipString:gsub("|NOT_ON_QUEST|","");
								self["QuestFrame"..TotalQuests].Tooltip = ToolTipString;	
								
								local Color = "FF00FF00";
								if OnQuest < getn(UnitsToCheck) then 
									Color = "FFFF0000";
								end 
								self["QuestFrame"..TotalQuests].LText:SetText(PrefixString.."[|c"..Color..OnQuest.."|r/"..getn(UnitsToCheck).."] "..QuestInfo.Name);
								if QuestInfo.Pushable == "1" then 
									if QuestInfo.Status == "M" then 
										self["QuestFrame"..TotalQuests].RText:SetText("[Request quest]");
									elseif QuestInfo.Status == "" and OnQuest < getn(UnitsToCheck) then  
										self["QuestFrame"..TotalQuests].RText:SetText("[Click to share]");
									else 
										self["QuestFrame"..TotalQuests].RText:SetText("");
									end 
								else 
									self["QuestFrame"..TotalQuests].RText:SetText("");
								end
								self["QuestFrame"..TotalQuests]:SetScript("OnClick",ShareQuest_OnClick);
								SetAnchor(self,"QuestFrame"..TotalQuests,LastFrame,QuestID)
								LastFrame = "QuestFrame"..TotalQuests;
							end 
						end 
					end 
				end 
			end 
			local FrameHeight = self.Header:GetHeight()/2 + self.RefreshButton:GetHeight() + 7.5;
			if TotalQuests > 0 then 
				FrameHeight = FrameHeight + TotalQuests*self["QuestFrame"..TotalQuests]:GetHeight();
			end 
			if TotalHeaders > 0 then 
				FrameHeight = FrameHeight + TotalHeaders*self["QuestHeader"..TotalHeaders]:GetHeight();
			end 
			self:SetHeight(FrameHeight);
		end 
		self.RefreshButton:Enable();
		SetAutoRefreshScript(self);
		LastUpdateTime = GetTime();
		self:Show();
	end 
end

local function UpdateParameters(self,ForceReset)
	local x,y = _Questic.GetVariable("GroupTrackPositionx"), _Questic.GetVariable("GroupTrackPositiony");
	if ForceReset or not x or not y then 
		x,y = UIParent:GetCenter();
		x = (x - self:GetWidth()/2);
		y = (y + self:GetHeight()/2);
	else 
		local PrevScale = self:GetScale();
		x = x*PrevScale;
		y = y*PrevScale;
	end 
	local NewScale = _Questic.GetVariable("CompareScale");
	self:SetScale(NewScale);
	x = x/NewScale;
	y = y/NewScale;
	self:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",x,y);
	if x ~= _Questic.GetVariable("GroupTrackPositionx") then 
		_Questic_ADDON_Globals.SaveVariable("GroupTrackPositionx",x)
	end 
	if _Questic.GetVariable("GroupTrackPositiony") ~= y then 
		_Questic_ADDON_Globals.SaveVariable("GroupTrackPositiony",y)
	end 
end 

_Questic.GroupTrack_OnLoad = function(self)
	Pool:CreatePool("BUTTON",self,"Questic_ADDON_GroupTrackQuestHeaderTemplate");
	Pool:CreatePool("BUTTON",self,"Questic_ADDON_GroupTrackQuestTemplate");
	_Questic_ADDON_Globals.SetBackdrop(self);
	_Questic_ADDON_Globals.SetBackdrop(self.Header);
	self.TimerValue = 0;
	
	self.Preview = function(self,Preview) 
		self.ActivePreview = Preview;
		Questic_ADDON_GroupTrack:Refresh(true);
	end 
	
	self.Refresh = GroupTrackRefresh;
	self.UpdateParameters = UpdateParameters;
	self.PartyQuestLogSync = PartyQuestLogSync;
	self.SetAutoRefreshScript = SetAutoRefreshScript; 
	
	if _Questic.GetVariable("GroupTrackPositionx") and _Questic.GetVariable("GroupTrackPositiony") then 
		self:UpdateParameters(); 
	end
end

_Questic.GroupTrack_OnShow_OnHide = function(self,Show)
	if Show then 
		Questic_ADDON_GroupTrack:Refresh();
	else 
		Pool:ReleaseAll();
	end 
end


