local _Questic = _Questic_ADDON_Globals;

local Pool = CreatePoolCollection();

local function UpdateItemSelect(self,Data)
	if Data then 
		if type(Data.Preview) == "boolean" then 
			self.Preview = Data.Preview; 
		end 
		if Data.Items then 
			self.Title = Data.Title or "Reward select";
			self.Items = Data.Items;
			self.OnConfirm = Data.OnConfirm;
			self.OnCancel = Data.OnCancel;
			self.RewardOnClick = Data.RewardOnClick;
		end 
	end 
	
	if (GetRewardText() == "" and GetProgressText() == "" and not self.Preview) or not self.Items then 
		return self:Hide();
	end 
	
	self.ConfirmButton:SetScript("OnClick",self.OnConfirm);
	self.ConfirmButton:HookScript("OnClick",function(self) self:GetParent():Hide() end);
	self.CancelButton:SetScript("OnClick",self.OnCancel);
	self.CancelButton:HookScript("OnClick",function(self) self:GetParent():Hide() end);
	if self.OnConfirm then 
		self.ConfirmButton:Show();
	else 
		self.ConfirmButton:Hide();
	end 
	self.Header.Text:SetText(self.Title or "Reward select");
	Pool:ReleaseAll();
	self.RewardButtons = {};
	for i,v in ipairs(self.Items) do 
		local RewardButton = Pool:Acquire("Questic_ADDON_ItemSelectButtonTemplate");
		RewardButton.ItemSubType = v.SubType; 
		RewardButton.ItemIndex = v.ID;
		RewardButton.ItemType = v.Type;
		RewardButton.OnClick = self.RewardOnClick;
		if ceil(i/2) ~= i/2 then 
			RewardButton:SetPoint("TOPLEFT",self,"TOPLEFT",_Questic.Padding + _Questic.BorderSize,-(self.Header:GetHeight()/2 + (ceil(i/2) - 1)*(RewardButton:GetHeight() + _Questic.Padding) + _Questic.Padding));
		else 
			RewardButton:SetPoint("TOPLEFT",self,"TOPLEFT",2*_Questic.Padding + _Questic.BorderSize + RewardButton:GetWidth(),-(self.Header:GetHeight()/2 + (ceil(i/2) - 1)*(RewardButton:GetHeight() + _Questic.Padding) + _Questic.Padding));
		end 
		RewardButton:Show();
		table.insert(self.RewardButtons,RewardButton);
	end 
	if getn(self.RewardButtons) > 0 then 
		local VertBorder = self.CancelButton:GetHeight() + _Questic.Padding + self.Header:GetHeight()/2;
		if self.ConfirmButton:IsShown() then 
			VertBorder = VertBorder + self.ConfirmButton:GetHeight() + _Questic.Padding;
		end
		self:SetHeight(ceil(getn(self.RewardButtons)/2)*(self.RewardButtons[1]:GetHeight() + _Questic.Padding) + VertBorder + _Questic.Padding + _Questic.BorderSize);
		self:Show();
	else 
		self:Hide();
	end
end 

local function UpdateParameters(self,ForceReset)
	local x,y = _Questic.GetVariable("RewardMenuPositionx"), _Questic.GetVariable("RewardMenuPositiony");
	if ForceReset or not x or not y then 
		x,y = UIParent:GetCenter();
		x = (x - self:GetWidth()/2);
		y = (y + self:GetHeight()/2);
	else 
		local PrevScale = self:GetScale();
		x = x*PrevScale;
		y = y*PrevScale;
	end 
	local NewScale = _Questic.GetVariable("RewardMenuScale");
	self:SetScale(NewScale);
	x = x/NewScale;
	y = y/NewScale;
	self:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",x,y);
	if x ~= _Questic.GetVariable("RewardMenuPositionx") then 
		_Questic_ADDON_Globals.SaveVariable("RewardMenuPositionx",x)
	end 
	if _Questic.GetVariable("RewardMenuPositiony") ~= y then 
		_Questic_ADDON_Globals.SaveVariable("RewardMenuPositiony",y)
	end 
end 

_Questic.ItemSelectFrame_OnHide = function(self)
	self.Items = nil; 
	self.Title = nil; 
	self.OnConfirm = nil; 
	self.OnCancel = nil; 
	self.RewardOnClick = nil;
	Pool:ReleaseAll();
	self.RewardButtons = {};
end

_Questic.ItemSelectFrame_OnLoad = function(self)
	Pool:CreatePool("FRAME",self,"Questic_ADDON_ItemSelectButtonTemplate");
	_Questic_ADDON_Globals.SetBackdrop(self);
	_Questic_ADDON_Globals.SetBackdrop(self.Header);
	self:RegisterForDrag("LeftButton");
	self.RewardButtons = {};
	
	self.UpdateParameters = UpdateParameters;
	self.UpdateItemSelect = UpdateItemSelect;
	
	if _Questic.GetVariable("RewardMenuPositionx") and _Questic.GetVariable("RewardMenuPositiony") then 
		self:UpdateParameters(); 
	end
end 