local _Questic = _Questic_ADDON_Globals;
local Pool = CreatePoolCollection();

local function SetOptions(self,Table,Key,AnchorCords,ForceHide)
	if type(Key) == "string" and type(self.Key) == "string" and Key == self.Key and (self:IsShown() or ForceHide) then 
		self:Hide();
		return false;
	end 
	
	if type(Table) ~= "table" or type(Key) ~= "string" then --ignore
		return false;
	end 
	
	if ForceHide then return false end; 
	
	self:Hide();
	
	if AnchorCords and AnchorCords.Point and AnchorCords.Ref and AnchorCords.RefPoint and AnchorCords.X and AnchorCords.Y then 
		self:ClearAllPoints();
		self:SetPoint(AnchorCords.Point,AnchorCords.Ref,AnchorCords.RefPoint,AnchorCords.X,AnchorCords.Y)
	end
	
	_Questic.BuilderFunc(self,Table,nil,nil,10);
	
	self.Key = Key;
	self:Show();
	return true;
end

local function RefreshOptions(self,Table,Key,AnchorCords)
	if type(Key) ~= "string" or type(self.Key) ~= "string" or Key ~= self.Key or not self:IsShown() or type(Table) ~= "table" then 
		return false;
	end  
	
	self:Hide();
	_Questic.BuilderFunc(self,Table,nil,nil,10);
	if AnchorCords and AnchorCords.Point and AnchorCords.Ref and AnchorCords.RefPoint and AnchorCords.X and AnchorCords.Y then 
		self:ClearAllPoints();
		self:SetPoint(AnchorCords.Point,AnchorCords.Ref,AnchorCords.RefPoint,AnchorCords.X,AnchorCords.Y)
	end
	self.Key = Key;
	self:Show();
	return true;
end

_Questic.MinimapDropDown_OnLoad = function(self)
	Pool:CreatePool("FRAME",self,"Questic_ADDON_OptionsCheckboxTemplate");
	Pool:CreatePool("FRAME",self,"Questic_ADDON_OptionsCatagoryTemplate");
	Pool:CreatePool("BUTTON",self,"Questic_ADDON_OptionsButtonTemplate");
	Pool:CreatePool("FRAME",self,"Questic_ADDON_OptionsNumericSelectTemplate");
	
	_Questic_ADDON_Globals.SetBackdrop(self);
	
	self.SetOptions = SetOptions;
	self.RefreshOptions = RefreshOptions;
end

local function HideOptions()
    if not self:IsMouseOver() then
		self.LeaveTime = GetTime();
		self:SetScript("OnUpdate",function(self,elapsed)
			if abs(self.LeaveTime - GetTime()) > 10 then
				self:Hide()
			end
		end)
    end		
end 

_Questic.MinimapDropDownItem_OnLeave_OnEnter = function(self,Enter)

end 

_Questic.MinimapDropDownItem_OnHide_OnShow = function(self,Show)
	if Show then 
	else 
		_Questic.ReleaseFunc(self);
		self.Key = nil;
	end 
end 

_Questic.AttachTooltipToOptions = function(Text,r,g,b,a,wrap,Frame)
	local AnchorFrame = Frame;
	for i = 1,100 do 
		if AnchorFrame:GetParent() then 
			if AnchorFrame:GetParent() == UIParent then 
				break;
			end 
			AnchorFrame = AnchorFrame:GetParent();
			if AnchorFrame.ThisAnchor then
				break; 
			end 
		end 
	end 
	if (not AnchorFrame.ThisAnchor and abs(Frame:GetLeft() - AnchorFrame:GetLeft()) < abs(Frame:GetRight() - AnchorFrame:GetRight())) or (AnchorFrame.ThisAnchor and AnchorFrame.ThisAnchor == "LEFT") then 
		Questic_Tooltip:SetOwner(AnchorFrame,"ANCHOR_BOTTOMLEFT",-_Questic.Padding,Frame:GetTop() - AnchorFrame:GetBottom());
	else 
		Questic_Tooltip:SetOwner(AnchorFrame,"ANCHOR_BOTTOMRIGHT",_Questic.Padding,Frame:GetTop() - AnchorFrame:GetBottom());
	end 
	Questic_Tooltip:SetText(Text,r,g,b,a,wrap);
end 

local ExtendedGossipListHeaders = {};
local BuiltGossipHeaders = {};
local SelectedGossip; 
local function AddCategorizedItemsToList(self,TargetTable,SourceTable,ParentCategory)
	table.sort(SourceTable,function(a,b) 
		local aCat = a.Categories or "";
		aCat = aCat..a.NPC..a.Gossip
		local bCat = b.Categories or "";
		bCat = bCat..b.NPC..b.Gossip
		return aCat < bCat;
	end)
	for i,v in ipairs(SourceTable) do 
		local FullCategoriesString = ParentCategory
		if v.Categories and v.Categories:gsub(" ","") ~= "" then 
			FullCategoriesString = FullCategoriesString.."|"..v.Categories;
		end 
		local Categories = {strsplit("|",FullCategoriesString)};
		
		local Collapsed = false; 
		local CategoryString = ""
		for Catei,Catev in ipairs(Categories) do 
			if CategoryString == "" then 
				CategoryString = Catev;
			else 
				CategoryString = CategoryString.."|"..Catev;
			end 
			if not BuiltGossipHeaders[CategoryString] then 
				local EndMarker = "End."
				
				local LocalCategoryString = CategoryString;
				table.insert(TargetTable,{Type = "DB_BL", Text = Catev, Alignment = "LEFT",
				Indent = Catei, Header = true, Expanded = ExtendedGossipListHeaders[LocalCategoryString], 
				Func = function()
					ExtendedGossipListHeaders[LocalCategoryString] = not ExtendedGossipListHeaders[LocalCategoryString]; 
					self:Refresh(Tab,1)
				end,
				Tooltip = Catev,});	
				
				BuiltGossipHeaders[CategoryString] = Catei; 								
			end 
			if not ExtendedGossipListHeaders[CategoryString] then 
				Collapsed = true;
				break; 
			end 
			--check higher order categories
			local SubString = ""
			for checkcatei = 1,Catei do 
				if SubString == "" then 
					SubString = Categories[checkcatei];
				else 
					SubString = SubString.."|"..Categories[checkcatei];
				end 
				if not ExtendedGossipListHeaders[SubString] then 
					Collapsed = true;									
				end 
			end 
			if Collapsed then 
				break; 
			end 
		end 
		
		
		if not Collapsed then 
			local Tooltip = "NPC\n"..v.NPC.."\nGossip\n"..v.Gossip;
			if v.Note and v.Note ~= "" then 
				Tooltip = Tooltip.."\n\nNote\n"..v.Note;
			end 
			
			local Func;
			local Value;
			local SelectFunc;
			if ParentCategory == "Preset" then 
				Value = not _Questic.GetVariable("GossipAutoSelectDisabledPresets")[v.NPC..v.Gossip..v.Categories]
				Func = function(_,Value,userInput)
					if userInput then 
						_Questic.GetVariable("GossipAutoSelectDisabledPresets")[v.NPC..v.Gossip..v.Categories] = not Value;
					end 
				end
			else 
				Value = not v.Disable;
				if ParentCategory == "User defined blacklist" then 
					Func = function(_,Value,userInput)
						if userInput then 
							_Questic.GossipFunctions.ChangeGossip("Blacklist",i,"Disable",not Value);
							if SelectedGossip and SelectedGossip == i then 
								self:Refresh(Tab,2)
							end 
						end 
					end

				else 
					Func = function(_,Value,userInput)
						if userInput then 
							_Questic.GossipFunctions.ChangeGossip("UserDefined",i,"Disable",not Value);
							if SelectedGossip and SelectedGossip == i then 
								self:Refresh(Tab,2)
							end 
						end 
					end
				end 
				SelectFunc = function()
					SelectedGossip = i;
					self:Refresh(Tab)
				end;
			end 
			
			table.insert(TargetTable,{Type = "CB", Text = v.NPC..": "..v.Gossip, Alignment = "LEFT",
			Indent = BuiltGossipHeaders[FullCategoriesString], 
			Selected = SelectedGossip == i and (ParentCategory == "User defined" or ParentCategory == "User defined blacklist"),
			Value = Value,
			Tooltip = Tooltip,
			Func = Func,
			SelectFunc = SelectFunc});
		end 
	end 
end 


local SelectedProfile; 
local CopyProfile; 
local CurrentGossipData = {}; 
_Questic_ADDON_Globals.OptionsMenu_OnLoad = function(self)
	_Questic.SetBackdrop(self);
	_Questic.SetBackdrop(self.Header);
	
	self.Header:SetPoint("TOPLEFT",_Questic.BorderSize + _Questic.Padding,-(_Questic.BorderSize + _Questic.Padding));
	self.Header:SetPoint("TOPRIGHT",self.CloseButton,"TOPLEFT",-_Questic.Padding,0);
	
	self.CloseButton:SetPoint("TOPRIGHT",-(_Questic.BorderSize + _Questic.Padding),-(_Questic.BorderSize + _Questic.Padding))
	
	_Questic.SetBackdrop(self.TabDisplay);
	self.TabDisplay:SetPoint("TOPLEFT",self.Header,"BOTTOMLEFT",0,-_Questic.Padding);
	self.TabDisplay:SetPoint("BOTTOMLEFT",self,"BOTTOMLEFT",(_Questic.BorderSize + _Questic.Padding),(_Questic.BorderSize + _Questic.Padding))
	
	_Questic.SetBackdrop(self.Content);
	self.Content:SetPoint("TOPLEFT",self.TabDisplay,"TOPRIGHT",_Questic.Padding,0);
	self.Content:SetPoint("BOTTOMRIGHT",-(_Questic.BorderSize + _Questic.Padding),(_Questic.BorderSize + _Questic.Padding))
	
	self.UpdatePosition = function(self)
		local x,y = _Questic.GetVariable("OptionsMenuPositionx"),_Questic.GetVariable("OptionsMenuPositiony")
		if not x then 
			local cx,cy = UIParent:GetCenter();
			x = cx - self:GetWidth()/2
		end 
		if not y then 
			local cx,cy = UIParent:GetCenter();
			y = cy + self:GetHeight()/2;
		end 
		self:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",x,y);	
	end 
	
	
	self.AnchorPoints = function()
	
	end 
	
	if _Questic.GetVariable("OptionsMenuPositionx") and _Questic.GetVariable("OptionsMenuPositiony") then 
		self:UpdatePosition();
	end
	
	self.CurrentTab = "General";
	self.Refresh = function(self,Tab,Row)
		if not self:IsShown() then 
			_Questic.ReleaseFunc(self.Content,1);
			_Questic.ReleaseFunc(self.Content,2);
			_Questic.ReleaseFunc(self.TabDisplay);
		end 	
		if Questic_ADDON_OptionsDropDownMenu.Key ~= "QuesticQuickShareKey" then 
			Questic_ADDON_OptionsDropDownMenu:SetOptions(nil,Questic_ADDON_OptionsDropDownMenu.Key,nil,true);
		end 	
		Row = Row or 3; 
		if self.CurrentTab ~= Tab and (Tab == "GossipList" or Tab == "GossipBlackList") then 
			SelectedGossip = nil;
			table.wipe(CurrentGossipData);
		end 
		self.CurrentTab = Tab or self.CurrentTab;
		if Row == 3 or Row == 0 then 
			----------------------------------------------------------------------------------------------------------------------
			-------------------------------------------Tab display----------------------------------------------------------------
			----------------------------------------------------------------------------------------------------------------------
			_Questic.BuilderFunc(self.TabDisplay,{
				{Type = "DB", Text = "General", Alignment = "LEFT",
				Selected = self.CurrentTab == "General",
				Func = function(self) Questic_ADDON_OptionsMenu:Refresh("General"); end},
				{Type = "LB", Text = ""},
				{Type = "LB", Text = "Basics"},
				{Type = "DB", Text = "Quest", Alignment = "LEFT",
				Selected = self.CurrentTab == "Quest",
				Func = function(self) Questic_ADDON_OptionsMenu:Refresh("Quest"); end},
				{Type = "DB", Text = "Gossip", Alignment = "LEFT",
				Selected = self.CurrentTab == "Gossip",
				Func = function(self) Questic_ADDON_OptionsMenu:Refresh("Gossip"); end},
				{Type = "LB", Text = "Gossip lists"},
				{Type = "DB", Text = "Whitelist", Alignment = "LEFT",
				Selected = self.CurrentTab == "GossipList",
				Func = function(self) Questic_ADDON_OptionsMenu:Refresh("GossipList"); end},
				{Type = "DB", Text = "Blacklist", Alignment = "LEFT",
				Selected = self.CurrentTab == "GossipBlackList",
				Func = function(self) Questic_ADDON_OptionsMenu:Refresh("GossipBlackList"); end},
				{Type = "LB", Text = ""},
				{Type = "LB", Text = "Advanced"},
				{Type = "DB", Text = "Group", Alignment = "LEFT",
				Selected = self.CurrentTab == "Group",
				Func = function(self) Questic_ADDON_OptionsMenu:Refresh("Group"); end},
				{Type = "LB", Text = ""},
				{Type = "DB", Text = "Profiles", Alignment = "LEFT",
				Selected = self.CurrentTab == "Profile",
				Func = function(self) Questic_ADDON_OptionsMenu:Refresh("Profile"); end},
			})
		end 
		if self.CurrentTab == "Quest" then 
			----------------------------------------------------------------------------------------------------------------------
			-------------------------------------------Quest Settings-------------------------------------------------------------
			----------------------------------------------------------------------------------------------------------------------
			if Row == 3 or Row == 1 then 
				_Questic.BuilderFunc(self.Content,{
					{Type = "CB",Text = "Auto-Share", VarName = "AutoShare",
					Tooltip = "Questic will automatically share any sharable quests."},
					{Type = "CBS",Text = "Quest auto-accept", VarName = "AutoAccept",
					Tooltip = "Automatically accept availiable quests when talking to NPCs,other players sharing quests or when other players accept escort quests.",
					SubElements = {
						{Type = "CB",Text = "Normal quests", VarName = "AutoAcceptNormal", 
						Tooltip = "Automatically accept normal quests."},
						{Type = "CB",Text = "Daily quests", VarName = "AutoAcceptDaily", 
						Tooltip = "Automatically accept daily quests."},
						{Type = "CB",Text = "Weekly quests", VarName = "AutoAcceptWeekly", 
						Tooltip = "Automatically accept weekly quests."},
						{Type = "CB",Text = "Low level quests", VarName = "AutoAcceptLowLevel",
						Tooltip = "When multiple quests are available automatically accept low level quests ("..GetQuestGreenRange().." level below current character level)."},
						{Type = "CB",Text = "Quests in bag", VarName = "AutoAcceptQuestsInBag",
						Tooltip = "Automatically accepts quests given by items in the players bags."},
					}},
					{Type = "CBS",Text = "Quest auto-turnin", VarName = "AutoTurnin",
					Tooltip = "Turn in quests automatically. Quests for which rewards have to be selected a window will be opened to prompt the player to select a reward and turn in the quest with on click.",
					SubElements = {
						{Type = "CB",Text = "Quest items", VarName = "AutoTurninQuestItems",
						Tooltip = "Automatically turn in quest items for completed quests without requiring player verification."},
						{Type = "CB",Text = "Non-quest items", VarName = "AutoTurninNonQuestItems", 
						Tooltip = "Automatically turn in items that are not marked as quest items when needed to complete the quest without requiring player verification."},
						{Type = "CB",Text = "Currency", VarName = "AutoturninCurrency",
						Tooltip = "Automatically turn in currency (not gold) when needed to complete the quest without requiring player confirmation.\n\nFor example \124T1397630:0\124t Order Resources in exchange for \124T1604167:0\124t Seal of Broken Fate."},
						{Type = "CB",Text = "Money", VarName = "AutoturninMoney",
						Tooltip = "Automatically turn in gold (money) when needed to complete the quest without requiring player confirmation.\n\nFor example 2,000g in exchange for \124T1604167:0\124t Seal of Broken Fate."},
					}},
					{Type = "CB",Text = "Hide standard quest window", VarName = "HideBlizzQuestFrame",
					Tooltip = "Hides the standard (Blizzard) quest window.\n\nRequires auto accept and auto turn-in to be enabled",
					Disable = not (_Questic.GetVariable("AutoAccept") and _Questic.GetVariable("AutoTurnin")),},
				},1,2,nil,nil,true);
			end 
			if Row == 3 or Row == 2 then 
				_Questic.BuilderFunc(self.Content,{	
					{Type = "CBS",Text = "Enable useable quest item bar", VarName = "QuestItemBar",
					Tooltip = "Searches for useable quest items in the players bags and creates a bar for easy use.",
					SubElements = {
						{Type = "SB",Text = "Growth direction", VarName = "QuestItemBarGrowthDirection",
						Tooltip = "The growth direction of the quest item bar.",
						GetList = function() 
							return {{Type = "DB_BL", Text = "Right"},{Type = "DB_BL", Text = "Left"},{Type = "DB_BL", Text = "Down"},{Type = "DB_BL", Text = "Up"}};
						end},
						{Type = "NB",Text = "Scale", VarName = "QuestItemBarScale",
						Tooltip = "Scale of compare quest display.",
						Max = 2, Min = 0.1, Step = 0.05,
						Func = function() Questic_ADDON_UseQuestItemBar:UpdateParameters() end, 
						},
						{Type = "DB", Text = "Move bar", Alignment = "LEFT",
						Tooltip = "Adds sample quest items (no none are in the players bags) to the quest item bar for moving and resizing.",
						Func = function(self) if Questic_ADDON_UseQuestItemBar:IsShown() then 
								Questic_ADDON_UseQuestItemBar:Movable();
							else 
								Questic_ADDON_UseQuestItemBar:Movable(true);
							end end},
						{Type = "DB", Text = "Reset position", Alignment = "LEFT",
						Tooltip = "Resets the position of the quest item bar.",
						Func = function(self) Questic_ADDON_UseQuestItemBar:UpdateParameters(true) end},
						{Type = "CB",Text = "Enable quest item blacklist", VarName = "QuestItemBarEnableBlackList",
						Tooltip = "Enable the list of items that will not be displayed on the quest item bar."},
						{Type = "SB",Text = "Add item to list", 
						Tooltip = "Add quest item to the blacklist to no longer be displayed in the quest item bar.",
						GetValue = function() return "" end,
						Func = function(self,Value) _Questic.QuestItemBlackList("Add",Value) end,
						GetList = function() 
							local ReturnList = {}
							for i,v in pairs(_Questic.QuestItemList) do 
								table.insert(ReturnList,{Type = "DB_BL", Text = GetItemInfo(select(7,GetContainerItemInfo(v.BagID,v.SlotID)))});
							end 
							return ReturnList;
						end},
						{Type = "SB",Text = "Remove item from list", 
						Tooltip = "Remove quest item from the blacklist to again be displayed in the quest item bar.",
						GetValue = function() return "" end; 
						Func = function(self,Value) _Questic.QuestItemBlackList("Remove",Value)  end,
						GetList = function() 
							local ReturnList = {}
							for i,v in pairs(_Questic.GetVariable("QuestItemBlackList")) do 
								table.insert(ReturnList,{Type = "DB_BL", Text = i});
							end 
							return ReturnList;
						end},
					}},		
					{Type = "LB",Text = "Reward select/Item turn in window"},
					{Type = "NB",Text = "Scale", VarName = "RewardMenuScale",
					Tooltip = "Scale reward/turn-in window.",
					Max = 3, Min = 0.4, Step = 0.1,
					Func = function() Questic_ADDON_ItemSelectMainFrame:UpdateParameters(); end,
					},
					{Type = "DB", Text = "Show/hide preview frame", Alignment = "LEFT",
					Tooltip = "Shows or hides a sample reward select/item turn-in frame if no reward select/item turn-in frame is open.",
					Func = function(self) 
						if Questic_ADDON_ItemSelectMainFrame:IsShown() then 
							Questic_ADDON_ItemSelectMainFrame:UpdateItemSelect({Preview = false});
						else 
							Questic_ADDON_ItemSelectMainFrame:UpdateItemSelect({
								Items = {
									{ID = "item:30234:0:0:0:0:0:0:0",Type = "preview", SubType = "preview"},
									{ID = "item:154983:0:0:0:0:0:0:0",Type = "preview", SubType = "preview"},
									{ID = "item:154977:0:0:0:0:0:0:0",Type = "preview", SubType = "preview"},
									{ID = "item:154973:0:0:0:0:0:0:0",Type = "preview", SubType = "preview"},
									{ID = "item:154971:0:0:0:0:0:0:0",Type = "preview", SubType = "preview"},
									{ID = "item:155448:0:0:0:0:0:0:0",Type = "preview", SubType = "preview"},
									{ID = "item:158713:0:0:0:0:0:0:0",Type = "preview", SubType = "preview"},
								},
								Title = "Required items",
								Preview = true,
								OnCancel = function()
									
								end,
								OnConfirm = function()
									
								end,
							});
						end 
					end},
					{Type = "DB", Text = "Reset position", Alignment = "LEFT",
					Tooltip = "Resets the position of the reward select/item turn-in frame.",
					Func = function(self) 
						Questic_ADDON_ItemSelectMainFrame:UpdateParameters(true); 
					end},
				},2,2,nil,nil,true);
			end
		elseif self.CurrentTab == "Gossip" then 
			----------------------------------------------------------------------------------------------------------------------
			------------------------------------------Gossip Settings-------------------------------------------------------------
			----------------------------------------------------------------------------------------------------------------------
			AddItemData = {
				["NPC"] = "",
				["Gossip"] = "",				
				["Note"] = "",	
				["Disable"] = false,	
			};	
			if Row == 3 or Row == 1 then 
				_Questic.BuilderFunc(self.Content,{
					{Type = "CBS",Text = "Enable gossip quick select", VarName = "GossipQuickSelect",
					Tooltip = "Whenever a NPC has gossip options they are displayed next to the cursor for quick selection.",
					SubElements = {
						{Type = "NB",Text = "Scale", VarName = "GossipQuickSelectScale",
						Tooltip = "Scale of the gossip quick select window.",
						Max = 3, Min = 0.4, Step = 0.1,
						Func = function() Questic_ADDON_GossipQuickSelectFrame:UpdateParameters(); end, 
						},
						{Type = "DB", Text = "Show/hide gossip quick select preview", Alignment = "LEFT",
						Tooltip = "Shows or hides a sample gossip frame if no gossip frame is open.",
						Func = function(self) 
							Questic_ADDON_GossipQuickSelectFrame:UpdateGossip(not Questic_ADDON_GossipQuickSelectFrame:IsShown(),true);
						end},
						{Type = "CB",Text = "Hide standard gossip window", VarName = "HideBlizzGossipFrame",
						Tooltip = "Hides the standard (Blizzard) gossip window."},
						{Type = "CB",Text = "Include quests", VarName = "IncludeQuestsInGosseipSelect",
						Tooltip = "Includes quests in the gossip quick select menu."},
					}},
					{Type = "CBS",Text = "Enable auto-gossip", VarName = "GossipAutoSelect",
					Tooltip = "Automtically selects gossip if it is part of the auto-gossip list.",
					SubElements = {
						{Type = "CB",Text = "Prioritize skip options", VarName = "GossipAutoSelectPriorSkip",
						Tooltip = "The addon will prioritize gossip options that skip sections of the story.\n\nFor example when having completed the battle for Lordaeron scenario this becomes skipable which the addon will always do if the option is enabled."},
						{Type = "CB",Text = "Ignore skip options", VarName = "GossipAutoSelectIgnoreSkip",
						Tooltip = "The addon will ignore gossip options that skip sections of the story.\n\nFor example when having completed the battle for Lordaeron scenario this becomes skipable which the addon will always do if the option is enabled."},
						{Type = "CB",Text = "Auto select gossip option if only one is available.", VarName = "GossipAutoSelectSingleOptionGossip",
						Tooltip = "If the NPC presents to the player with only one gossip option this one gossip option will be selected."},
						{Type = "LB", Text = ""},
						{Type = "DB", Text = "Show auto-gossip whitelist", Alignment = "LEFT",
						Tooltip = "A list of NPCs for which set gossip options are automatically selected.",
						Func = function(self) 
							Questic_ADDON_OptionsMenu:Refresh("GossipList");
						end},
						{Type = "DB", Text = "Show auto-gossip blacklist", Alignment = "LEFT",
						Tooltip = "A list of gossip items for which set gossip options are not automatically selected (The blacklist overrides any other auto-gossip selection).",
						Func = function(self) 
							Questic_ADDON_OptionsMenu:Refresh("GossipBlackList");
						end},
					}},
					{Type = "CB",Text = "Right-Click gossip add", VarName = "RightClickGossipAdd",
					Tooltip = "When Right-Clicking a gossip option in the Blizzard or gossip quick select frame it is automatically added to the auto-gossip list."},
					{Type = "CB",Text = "Modified Right-Click gossip blacklist", VarName = "RightClickGossipBlacklistAdd",
					Tooltip = "When Right-Clicking while pressing a modifier key (alt, shift or ctrl) a gossip option in the Blizzard or gossip quick select frame it is automatically added to gossip blacklist."},
				},1,2,nil,nil,true);
			end 
			if Row == 3 or Row == 2 then 
				_Questic.BuilderFunc(self.Content,{},2,2,nil,nil,true);
			end
		elseif self.CurrentTab == "GossipList" then 
			----------------------------------------------------------------------------------------------------------------------
			---------------------------------------Gossip List Settings-----------------------------------------------------------
			----------------------------------------------------------------------------------------------------------------------
			if Row == 3 or Row == 1 then 
				local GossipListBuildTable = {};
				table.wipe(BuiltGossipHeaders);
				
				AddCategorizedItemsToList(self,GossipListBuildTable,_Questic.GetVariable("GossipAutoSelectList"),"User defined");
				
				AddCategorizedItemsToList(self,GossipListBuildTable,_Questic_ADDON_Globals.GossipFunctions.DefaultList,"Preset");
				
				_Questic.BuilderFunc(self.Content,GossipListBuildTable,1,2,19,self.Content["ScrollStartIndex1"],true);
			end 
			if Row == 3 or Row == 2 then 
				if SelectedGossip and _Questic.GetVariable("GossipAutoSelectList")[SelectedGossip] then 
					CurrentGossipData = {
						["NPC"] = _Questic.GetVariable("GossipAutoSelectList")[SelectedGossip]["NPC"],
						["Gossip"] = _Questic.GetVariable("GossipAutoSelectList")[SelectedGossip]["Gossip"],
						["Note"] = _Questic.GetVariable("GossipAutoSelectList")[SelectedGossip]["Note"],
						["Categories"] = _Questic.GetVariable("GossipAutoSelectList")[SelectedGossip]["Categories"],
						["Disable"] = _Questic.GetVariable("GossipAutoSelectList")[SelectedGossip]["Disable"],
					};
				end 
				
				_Questic.BuilderFunc(self.Content,{
					--add new entry 
					{Type = "DB", Text = "Add new entry",
					Func = function()
						CurrentGossipData = {
							["NPC"] = "",
							["Gossip"] = "",
							["Note"] = "",
						};
						SelectedGossip = nil;
						Questic_ADDON_OptionsMenu:Refresh(Tab);						
					end},
					
					--add last gossip
					{Type = "DB", Text = "Add last entry", Disable = _Questic.GossipFunctions.FetchLastGossip()["NPC"] == "",
					Func = function()
						if _Questic.GossipFunctions.FetchLastGossip()["NPC"] == "" then 
							print("Questic: No previous gossip item.");
						else 
							CurrentGossipData = {
								["NPC"] = _Questic.GossipFunctions.FetchLastGossip()["NPC"],
								["Gossip"] = _Questic.GossipFunctions.FetchLastGossip()["Gossip"],
								["Note"] = _Questic.GossipFunctions.FetchLastGossip()["Note"],
								["Categories"] = _Questic.GossipFunctions.FetchLastGossip()["Categories"],
								["Disable"] = _Questic.GossipFunctions.FetchLastGossip()["Disable"],
							};
							SelectedGossip = nil;
							Questic_ADDON_OptionsMenu:Refresh(Tab);								
						end 
					end},

					--category 1
					CurrentGossipData["NPC"] and {Type = "TB",Text = "Category", Tooltip = "The category to file the gossip.\n\n Seperate sub categories with \"|\" or \",\".\n\nFor example Cat1,Cat2 or Cat1|Cat2 will yield:\n\n|TInterface\\AddOns\\Questic\\Media\\ArrowExpanded:0|t Cat1\n   |TInterface\\AddOns\\Questic\\Media\\ArrowExpanded:0|t Cat2\n      The gossip item", 
					Value = (CurrentGossipData["Categories"] or ""):gsub("|",","),
					Func = function(self,Value,PrevValue,userInput)
						if userInput then 
							CurrentGossipData["Categories"] = Value:gsub(",","|"):gsub("||","|");
						end 
					end} or nil,
					
					--npc name 
					CurrentGossipData["NPC"] and {Type = "TB",Text = "NPC name", Tooltip = "Name of the NPC.", 
					Value = CurrentGossipData["NPC"] or "",
					Func = function(self,Value,PrevValue,userInput)
						if userInput then 
							CurrentGossipData["NPC"] = Value;
						end 
					end} or nil,
					--gossip text 
					CurrentGossipData["NPC"] and {Type = "TB",Text = "Gossip text", Tooltip = "Gossip text as shown in the gossip window.", 
					Lines = 2, 
					Value = CurrentGossipData["Gossip"] or "",
					Func = function(self,Value,PrevValue,userInput)
						if userInput then 
							CurrentGossipData["Gossip"] = Value;
						end
					end} or nil,
					
					--note 
					CurrentGossipData["NPC"] and {Type = "TB",Text = "Note (Optional)", Tooltip = "", 
					Lines = 5, 
					Value = CurrentGossipData["Note"] or "",
					Func = function(self,Value,PrevValue,userInput)
						if userInput then 
							CurrentGossipData["Note"] = Value;
						end 
					end} or nil,
					
					--active 
					CurrentGossipData["NPC"] and {Type = "CB",Text = "Active",
					Tooltip = "Whether or not the gossip should be auto-selected.", 
					Func = function(self,Value,userInput)
						if userInput then 
							CurrentGossipData["Disable"] = not Value;
						end 
					end,
					Value = not CurrentGossipData["Disable"]} or nil,
				
				
					SelectedGossip and {Type = "DB", Text = "Save changes",
					Func = function()
						_Questic.GossipFunctions.ChangeGossip("UserDefined",SelectedGossip,"NPC",CurrentGossipData["NPC"]);
						_Questic.GossipFunctions.ChangeGossip("UserDefined",SelectedGossip,"Gossip",CurrentGossipData["Gossip"]);
						_Questic.GossipFunctions.ChangeGossip("UserDefined",SelectedGossip,"Categories",CurrentGossipData["Categories"]);
						_Questic.GossipFunctions.ChangeGossip("UserDefined",SelectedGossip,"Note",CurrentGossipData["Note"]);
						_Questic.GossipFunctions.ChangeGossip("UserDefined",SelectedGossip,"Disable",CurrentGossipData["Disable"]);
						table.wipe(CurrentGossipData);
						Questic_ADDON_OptionsMenu:Refresh(Tab);
					end 
					} or CurrentGossipData["NPC"] and {Type = "DB", Text = "Add item",
					Func = function()
						_Questic.GossipFunctions.ChangeGossip("UserDefined",nil,"ADD",CurrentGossipData);
						table.wipe(CurrentGossipData);
						Questic_ADDON_OptionsMenu:Refresh(Tab);
					end 
					} or nil, 
					
					SelectedGossip and {Type = "DB", Text = "Discard changes",
					Func = function()
						Questic_ADDON_OptionsMenu:Refresh(Tab,2);							
					end 
					} or CurrentGossipData["NPC"] and {Type = "DB", Text = "Cancel",
					Func = function()
						table.wipe(CurrentGossipData);
						Questic_ADDON_OptionsMenu:Refresh(Tab,2);
					end 
					} or nil, 
					
					SelectedGossip and {Type = "LB", Text = ""} or nil,
					SelectedGossip and {Type = "DB", Text = "Delete",
					Func = function()
						_Questic.GossipFunctions.ChangeGossip("UserDefined",SelectedGossip,"REMOVE");
						SelectedGossip = nil;
						table.wipe(CurrentGossipData);
						Questic_ADDON_OptionsMenu:Refresh(Tab);
					end 
					} or nil, 
				},2,2,nil,nil,true);
			end
		elseif self.CurrentTab == "GossipBlackList" then 
			----------------------------------------------------------------------------------------------------------------------
			------------------------------------Gossip Blacklist Settings---------------------------------------------------------
			----------------------------------------------------------------------------------------------------------------------
			if Row == 3 or Row == 1 then 
				local GossipListBuildTable = {};
				table.wipe(BuiltGossipHeaders);
				
				AddCategorizedItemsToList(self,GossipListBuildTable,_Questic.GetVariable("GossipAutoSelectBlackList"),"User defined blacklist");
				
				_Questic.BuilderFunc(self.Content,GossipListBuildTable,1,2,19,self.Content["ScrollStartIndex1"],true);
			end 
			if Row == 3 or Row == 2 then 
				if SelectedGossip and _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedGossip] then 
					CurrentGossipData = {
						["NPC"] = _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedGossip]["NPC"],
						["Gossip"] = _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedGossip]["Gossip"],
						["Note"] = _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedGossip]["Note"],
						["Categories"] = _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedGossip]["Categories"],
						["Disable"] = _Questic.GetVariable("GossipAutoSelectBlackList")[SelectedGossip]["Disable"],
					};
				end 
				
				_Questic.BuilderFunc(self.Content,{
					--add new entry 
					{Type = "DB", Text = "Add new entry",
					Func = function()
						CurrentGossipData = {
							["NPC"] = "",
							["Gossip"] = "",
							["Note"] = "",
						};
						SelectedGossip = nil;
						Questic_ADDON_OptionsMenu:Refresh(Tab);						
					end},
					
					--add last gossip
					{Type = "DB", Text = "Add last entry", Disable = _Questic.GossipFunctions.FetchLastGossip()["NPC"] == "",
					Func = function()
						if _Questic.GossipFunctions.FetchLastGossip()["NPC"] == "" then 
							print("Questic: No previous gossip item.");
						else 
							CurrentGossipData = {
								["NPC"] = _Questic.GossipFunctions.FetchLastGossip()["NPC"],
								["Gossip"] = _Questic.GossipFunctions.FetchLastGossip()["Gossip"],
								["Note"] = _Questic.GossipFunctions.FetchLastGossip()["Note"],
								["Categories"] = _Questic.GossipFunctions.FetchLastGossip()["Categories"],
								["Disable"] = _Questic.GossipFunctions.FetchLastGossip()["Disable"],
							};
							SelectedGossip = nil;
							Questic_ADDON_OptionsMenu:Refresh(Tab);								
						end 
					end},

					--category 1
					CurrentGossipData["NPC"] and {Type = "TB",Text = "Category", Tooltip = "The category to file the gossip.\n\n Seperate sub categories with \"|\" or \",\".\n\nFor example Cat1,Cat2 or Cat1|Cat2 will yield:\n\n|TInterface\\AddOns\\Questic\\Media\\ArrowExpanded:0|t Cat1\n   |TInterface\\AddOns\\Questic\\Media\\ArrowExpanded:0|t Cat2\n      The gossip item", 
					Value = (CurrentGossipData["Categories"] or ""):gsub("|",","),
					Func = function(self,Value,PrevValue,userInput)
						if userInput then 
							CurrentGossipData["Categories"] = Value:gsub(",","|"):gsub("||","|");
						end 
					end} or nil,
					
					--npc name 
					CurrentGossipData["NPC"] and {Type = "TB",Text = "NPC name", Tooltip = "Name of the NPC.", 
					Value = CurrentGossipData["NPC"] or "",
					Func = function(self,Value,PrevValue,userInput)
						if userInput then 
							CurrentGossipData["NPC"] = Value;
						end 
					end} or nil,
					--gossip text 
					CurrentGossipData["NPC"] and {Type = "TB",Text = "Gossip text", Tooltip = "Gossip text as shown in the gossip window.", 
					Lines = 2, 
					Value = CurrentGossipData["Gossip"] or "",
					Func = function(self,Value,PrevValue,userInput)
						if userInput then 
							CurrentGossipData["Gossip"] = Value;
						end
					end} or nil,
					
					--note 
					CurrentGossipData["NPC"] and {Type = "TB",Text = "Note (Optional)", Tooltip = "", 
					Lines = 5, 
					Value = CurrentGossipData["Note"] or "",
					Func = function(self,Value,PrevValue,userInput)
						if userInput then 
							CurrentGossipData["Note"] = Value;
						end 
					end} or nil,
					
					--active 
					CurrentGossipData["NPC"] and {Type = "CB",Text = "Active",
					Tooltip = "Whether or not the gossip should be auto-selected.", 
					Func = function(self,Value,userInput)
						if userInput then 
							CurrentGossipData["Disable"] = not Value;
						end 
					end,
					Value = not CurrentGossipData["Disable"]} or nil,
				
				
					SelectedGossip and {Type = "DB", Text = "Save changes",
					Func = function()
						_Questic.GossipFunctions.ChangeGossip("Blacklist",SelectedGossip,"NPC",CurrentGossipData["NPC"]);
						_Questic.GossipFunctions.ChangeGossip("Blacklist",SelectedGossip,"Gossip",CurrentGossipData["Gossip"]);
						_Questic.GossipFunctions.ChangeGossip("Blacklist",SelectedGossip,"Categories",CurrentGossipData["Categories"]);
						_Questic.GossipFunctions.ChangeGossip("Blacklist",SelectedGossip,"Note",CurrentGossipData["Note"]);
						_Questic.GossipFunctions.ChangeGossip("Blacklist",SelectedGossip,"Disable",CurrentGossipData["Disable"]);
						table.wipe(CurrentGossipData);
						Questic_ADDON_OptionsMenu:Refresh(Tab);
					end 
					} or CurrentGossipData["NPC"] and {Type = "DB", Text = "Add item",
					Func = function()
						_Questic.GossipFunctions.ChangeGossip("Blacklist",nil,"ADD",CurrentGossipData);
						table.wipe(CurrentGossipData);
						Questic_ADDON_OptionsMenu:Refresh(Tab);
					end 
					} or nil, 
					
					SelectedGossip and {Type = "DB", Text = "Discard changes",
					Func = function()
						Questic_ADDON_OptionsMenu:Refresh(Tab,2);							
					end 
					} or CurrentGossipData["NPC"] and {Type = "DB", Text = "Cancel",
					Func = function()
						table.wipe(CurrentGossipData);
						Questic_ADDON_OptionsMenu:Refresh(Tab,2);
					end 
					} or nil, 
					
					SelectedGossip and {Type = "LB", Text = ""} or nil,
					SelectedGossip and {Type = "DB", Text = "Delete",
					Func = function()
						_Questic.GossipFunctions.ChangeGossip("Blacklist",SelectedGossip,"REMOVE");
						SelectedGossip = nil;
						table.wipe(CurrentGossipData);
						Questic_ADDON_OptionsMenu:Refresh(Tab);
					end 
					} or nil, 
				},2,2,nil,nil,true);
			end 
		elseif self.CurrentTab == "Group" then 
			----------------------------------------------------------------------------------------------------------------------
			-------------------------------------------Group Settings-------------------------------------------------------------
			----------------------------------------------------------------------------------------------------------------------
			if Row == 3 or Row == 1 then 
				_Questic.BuilderFunc(self.Content,{
					{Type = "CBS",Text = "Enable group tracker", VarName = "GroupTrack",
					Tooltip = "The group track compares the players questlog with that of other group members and allows the sharing of quests with one click (reduced functionality if the other player does not have Questic).",
					SubElements = {
						{Type = "CB",Text = "Enable in groups", VarName = "QuestCompareInGroup",
						Tooltip = "Show group quest compare window when in groups."},
						{Type = "CB",Text = "Enable in raid groups", VarName = "QuestCompareInRaid",
						Tooltip = "Show group quest compare window when in raid groups."},
						{Type = "CB",Text = "Timed-refresh", VarName = "QuestCompareAutoRefresh",
						Tooltip = "Periodically refreshes the quest compare frame."},
						{Type = "NB",Text = "Refresh rate (in seconds)", VarName="CompareFrequency",
						Tooltip = "The frequency at which the compare display is refreshed.",
						Max = 60, Min = 1, Step = 1},
						{Type = "NB",Text = "Scale", VarName = "CompareScale",
						Tooltip = "Scale of compare quest display.",
						Max = 2, Min = 0.1, Step = 0.05,
						Func = function() Questic_ADDON_GroupTrack:UpdateParameters(); end, 
						},
						{Type = "DB", Text = "Show/hide preview frame", Alignment = "LEFT",
						Tooltip = "Shows or hides a sample group track frame if no group track frame is open.",
						Func = function(self) Questic_ADDON_GroupTrack:Preview(not Questic_ADDON_GroupTrack.ActivePreview); end},
						{Type = "DB", Text = "Reset position", Alignment = "LEFT",
						Tooltip = "Resets the position of the group track frame.",
						Func = function(self) Questic_ADDON_GroupTrack:UpdateParameters(true); end},
					}},
				},1,2,nil,nil,true);
			end 
			if Row == 3 or Row == 2 then 
				_Questic.BuilderFunc(self.Content,{},2,2,nil,nil,true);
			end
		elseif self.CurrentTab == "Profile" then
			----------------------------------------------------------------------------------------------------------------------
			-----------------------------------------Profile Settings-------------------------------------------------------------
			----------------------------------------------------------------------------------------------------------------------
			if Row == 3 or Row == 1 then 
				local Profiles = _Questic.ProfileManagement("GetAll");
				local ProfileButtons = {}
				for i,v in ipairs(Profiles) do 
					table.insert(ProfileButtons,{Type = "DB_BL", Text = v, Alignment = "CENTER",
					Selected = v == SelectedProfile, 
					Func = function(self)
						SelectedProfile = v; 
						Questic_ADDON_OptionsMenu:Refresh();
					end})
				end 
				_Questic.BuilderFunc(self.Content,ProfileButtons,1,2,16,self.Content["ScrollStartIndex1"],true);
			end 
			if Row == 3 or Row == 2 then 
				_Questic.BuilderFunc(self.Content,{
					{Type = "LB", Text = "Active profile: "..(_Questic.ProfileManagement("GetActive") or "")},
					{Type = "LB", Text = ""},
					{Type = "DB", Text = "Create new profile", Alignment = "CENTER",
					Func = function(self)
						SelectedProfile = _Questic.ProfileManagement("CreateNew") 
						Questic_ADDON_OptionsMenu:Refresh();
					end},
					{Type = "LB", Text = ""},
					{Type = "LB", Text = "Selected profile: "..(SelectedProfile or "")},
					{Type = "DB", Text = "Load", Alignment = "CENTER", Disable = not SelectedProfile,
					Func = function(self)
						SelectedProfile = _Questic.ProfileManagement("Load",SelectedProfile);
						Questic_ADDON_OptionsMenu:Refresh();							
					end},
					{Type = "LB", Text = ""},
					{Type = "TB",Text = "Change selected profile name", Disable = not SelectedProfile,
					Tooltip = "", Lines = 1,
					Func = function(self,Value,PrevValue,userInput)
						if userInput and Value ~= PrevValue then 
							SelectedProfile = _Questic.ProfileManagement("Rename",SelectedProfile,Value);
							Questic_ADDON_OptionsMenu:Refresh();
						end 
					end, 
					GetValue = function() return SelectedProfile or "" end},
					{Type = "LB", Text = ""},
					{Type = "SB",Text = "Copy settings from:", Disable = not SelectedProfile,
					GetList = function() 
						local Profiles = _Questic.ProfileManagement("GetAll");
						local ProfileButtons = {}
						for i,v in ipairs(Profiles) do 
							if v ~= SelectedProfile then 
								table.insert(ProfileButtons,{Type = "DB_BL", Text = v, Alignment = "CENTER",
								Func = function(self)
									CopyProfile = v; 
									Questic_ADDON_OptionsMenu:Refresh();		
								end})
							end 
						end 
						return ProfileButtons;
					end,
					GetValue = function() 
						return CopyProfile or "";
					end,},
					{Type = "DB", Text = "Copy profile settings", Alignment = "CENTER", Disable = not (SelectedProfile and CopyProfile),
					Tooltip = "Copy settings from \""..(CopyProfile or "").."\" to \""..(SelectedProfile or "").."\".",
					Func = function(self)
						_Questic.ProfileManagement("Copy",SelectedProfile,CopyProfile);
						CopyProfile = nil; 
						Questic_ADDON_OptionsMenu:Refresh();				
					end},
					{Type = "LB", Text = ""},
					{Type = "DB", Text = "Delete", Alignment = "CENTER", Disable = not SelectedProfile,
					Func = function(self)
						SelectedProfile = _Questic.ProfileManagement("Delete",SelectedProfile);
						CopyProfile = nil; 
						Questic_ADDON_OptionsMenu:Refresh();				
					end},
				},2,2,nil,nil,true);
			end
		else 
			----------------------------------------------------------------------------------------------------------------------
			-----------------------------------------General Settings-------------------------------------------------------------
			----------------------------------------------------------------------------------------------------------------------
			if Row == 3 or Row == 1 then 
				_Questic.BuilderFunc(self.Content,{
					{Type = "CB",Text = "Enable", VarName = "Enable",
					Tooltip = "Enable or disable all of Questic's functionality."},
					{Type = "CBS",Text = "Enable Questic suppress keys", VarName = "SuppressKeys", 
					Tooltip = "Suppress Questic (Auto-Accept, turnin and cinematic skip) when any key selected below is pressed.",
					SubElements = {
						{Type = "CB",Text = "Alt-key Suppress", VarName = "AltSuppress",
						Tooltip = "Suppress Questic when the Alt key is pressed."},
						{Type = "CB",Text = "Shift-key Suppress", VarName = "ShiftSuppress",
						Tooltip = "Suppress Questic when the Shift key is pressed."},
						{Type = "CB",Text = "CTRL-key Suppress", VarName = "CTRLSuppress",
						Tooltip = "Suppress Questic when the CTRL key is pressed."},
					}},
				},1,2,nil,nil,true);
			end 
			if Row == 3 or Row == 2 then 
				_Questic.BuilderFunc(self.Content,{	
					{Type = "CB",Text = "Enable minimap button", VarName = "EnableMinimapIcon",
					Tooltip = "Enable or disable Questic's minimap button."},
					{Type = "LB",Text = "Cinematic skip"},
					{Type = "CB",Text = "Enable", VarName = "CinematicSkip",
					Tooltip = "Automatically skipping cinematics, cutscenes or ingame movies."},
				},2,2,nil,nil,true);
			end
		end 		
		if self.CurrentTab ~= "Gossip" and self.CurrentOptionsCallListType == "AutoGossipList" then 
			self.CurrentOptionsCallListType = nil; 
		end 
	end;
	self.Call = function(self)
		if self:IsShown() then 
			self:Hide();
		else 
			self:Show();
		end 
	end 
end 

_Questic_ADDON_Globals.OptionsMenu_OnShow_OnHide = function(self,Show)
	if Show then 
		self:Refresh();
	else 
		Questic_ADDON_ItemSelectMainFrame:UpdateItemSelect({Preview = false});
		Questic_ADDON_GossipQuickSelectFrame:UpdateGossip(false,true);
		Questic_ADDON_GroupTrack:Preview(false);
		Questic_ADDON_UseQuestItemBar:Movable();
		
		
		_Questic.ReleaseFunc(self.Content,1);
		_Questic.ReleaseFunc(self.Content,2);
		_Questic.ReleaseFunc(self.TabDisplay);
		
		if Questic_ADDON_OptionsDropDownMenu.Key ~= "QuesticQuickShareKey" then 
			Questic_ADDON_OptionsDropDownMenu:SetOptions(nil,Questic_ADDON_OptionsDropDownMenu.Key,nil,true);
		end 
	end 
end



