local _Questic = _Questic_ADDON_Globals;

---------------------------------------------------------------
--Options Buttons
---------------------------------------------------------------
_Questic.OptionsDropDownTexture_OnLoad = function(self)
	self.Texture:SetColorTexture(1,0.82,0);
	self.Texture:Hide();
	self.MouseOverHighlight = function(self,ForceLeave)
		if not self.Selected then 
			self:SetScript("OnUpdate",function(self)
				if MouseIsOver(self) and not ForceLeave then 
					self.Texture:SetGradientAlpha("HORIZONTAL", 1,0.82,0, 0.6, 1,0.82,0, 0);
					self.Texture:Show();
				else 
					self.Texture:Hide();
					self:SetScript("OnUpdate",nil);
				end 
			end)
		end 
	end
	self.ClickHighlight = function(self,Click)
		if Click and not self.Selected then 
			self:SetScript("OnUpdate",nil);
			self.Texture:SetGradientAlpha("HORIZONTAL", 1,0.82,0, 1, 1,0.82,0, 0);
			self.Texture:Show();
		else 
			self:MouseOverHighlight();
		end 
	end
	self.SelectHighlight = function(self,Selected)
		if Selected then 
			self:SetScript("OnUpdate",nil);
			self.Texture:SetGradientAlpha("HORIZONTAL", 1,1,1, 1, 1,1,1, 0);	
			self.Texture:Show();		
			self.Selected = true; 
		else 
			self.Selected = nil; 
			self:MouseOverHighlight();
		end 
	end 
	
	if GetLocale() == "ruRU" and self.Text then 
		local Font,Size,Flag = self.Text:GetFont();
		self.Text:SetFont("Fonts\\FRIZQT___CYR.TTF",Size,Flag);
	end 
end

_Questic.OptionsDropDownFrame_OnEnter_OnLeave = function(self,Enter)
	self:MouseOverHighlight(not Enter);
	if Enter then 
		if type(self.Tooltip) == "string" then
			_Questic.AttachTooltipToOptions(self.Tooltip,1,1,1,1,true,self);
		end 
	else 
		Questic_Tooltip:Hide();
	end 
end 

local function NumericSetValue(self,Value,Increment,userInput)
	if type(Value) ~= "number" then 
		self.EditBox:ClearFocus(); 
		self.EditBox:SetText(self.Value);
		return false;
	end 
	local PrevVal = self.Value;
	if Increment then 
		self.Value = self.Value + Value*self.Step;
	else 
		self.Value = Value
	end 
	if self.Value > self.Max then 
		self.Value = self.Max;
	elseif self.Value < self.Min then 
		self.Value = self.Min;
	elseif floor(self.Value/self.Step) ~= self.Value/self.Step then 
		self.Value = _Questic.Round(self.Value/self.Step)*self.Step; 
	end 
	self.EditBox:ClearFocus(); 
	self.EditBox:SetText(self.Value);
	if type(self.OnValueChanged) == "function" then 
		self:OnValueChanged(self.Value,PrevVal,userInput);
	end 
end 

local function NumericSetMinMaxStep(self,Min,Max,Step)
	if type(Min) == "number" then 
		self.Min = Min;
	end 
	if type(Max) == "number" then 
		self.Max = Max;	
	end 
	if type(Step) == "number" then 
		self.Step = Step;
	end 
	if self.Min > self.Max then 
		local Mintemp = self.Min;
		self.Min = self.Max;
		self.Max = Mintemp;
	end
end 

_Questic.OptionsNumericSelect_OnLoad = function(self)
	self.Value = 0;	self.Min = 0; self.Max = 10; self.Step = 1;
	self.SetValue = NumericSetValue;
	self.SetMinMaxStep = NumericSetMinMaxStep; 
	self.EditBox:SetPoint("TOPRIGHT",self.Minus,"TOPLEFT");
	self.EditBox:SetPoint("BOTTOMRIGHT",self.Minus,"BOTTOMLEFT");
	self.Minus:SetPoint("RIGHT",self.Plus,"LEFT",-_Questic.Padding,0);
	self:SetValue();
end 

--OPTIONS MENU SCRIPTS

_Questic.Questic_ADDON_OptionsSectionTamplate_OnLoad = function(self)
		
	self.AddItems = function(self,DataTable,Key,MaxItems)
		self.Key = Key;
		self.BuildTable = DataTable;
		self.MaxItems = MaxItems;
	end 
	
	self.Refresh = function(self)
		if #self.BuildTable > 0 and self.Header.Checkbox:GetChecked() and self.BuildTable then 
			_Questic.BuilderFunc(self,self.BuildTable,1,1,self.MaxItems);
		else 
			_Questic.ReleaseFunc(self);
			self:SetHeight(self.Header:GetHeight());
		end 
		_Questic.RedoOptionHeight(self:GetParent());
	end 
	
	self.SetValue = function(self,Value)
		self.Header.Checkbox:SetChecked(Value);		
		self:Refresh();
	end 
	
	self.Header.TTAnchor = self:GetParent(); 
	
	self.Header:SetPoint("TOPLEFT");
	self.Header:SetPoint("TOPRIGHT");
	self.Header:SetHeight(self.Header.Checkbox:GetHeight() + 2*_Questic.Padding);
	self:SetHeight(self.Header:GetHeight());
	
	self.Header:SetScript("OnEnter",function(self)
		if self:GetParent().Tooltip then 
			_Questic.AttachTooltipToOptions(self:GetParent().Tooltip,1,1,1,1,true,self:GetParent());
		end 
		_Questic_ADDON_Globals.HighlightBorder(self);
	end)
	
	self.Header:SetScript("OnLeave",function(self)
		Questic_Tooltip:Hide(); 
		_Questic_ADDON_Globals.HighlightBorder(self,false,true);
	end)
	
	_Questic.SetBackdrop(self);
	_Questic.SetBackdrop(self.Header);
end 

_Questic.Questic_ADDON_OptionsSectionTamplate_OnClick = function(self,Type,UserInput)
	if Type == "Check" then 
		if self.Header:IsMouseOver() or self.Header.Checkbox:IsMouseOver() then
			if type(self.Header.OnChecked) == "function" then
				self.Header.OnChecked(self,self.Header.Checkbox:GetChecked(),UserInput);
			end
			self:Refresh();
        end
	elseif Type == "Minize" then  
		self:Refresh()
	end 

end 

local function Clear(self)
	_Questic.ReleaseFunc(self)
	self.StartIndex = 1; 
end 

local ListBoxBuildTable = {};
local function Insert(self,Table,MaxIndex)
	self.Data = Table or self.Data; 
	self.MaxIndex = MaxIndex or #self.Data;
	
	table.wipe(ListBoxBuildTable); 
	
	local DisplayedItem = 0
	for i = self.StartIndex,#self.Data do 
		table.insert(ListBoxBuildTable,self.Data[i]);		
		DisplayedItem = DisplayedItem + 1; 
		if DisplayedItem >= self.MaxIndex then 
			break; 
		end 
	end 
	
	_Questic.BuilderFunc(self,Table);
	
	--scrolbar adjustment
	
	
end 

_Questic_ADDON_Globals.Questic_ADDON_OptionsListBoxTamplate_OnLoad = function(self)
	self.Data = {};
	_Questic.SetBackdrop(self);
	
	self.StartIndex = 1;
	self.MaxIndex = 1; 
	
	self.Insert = Insert;
	self.Clear = Clear;
end 

_Questic_ADDON_Globals.Questic_ADDON_OptionsEditBoxTamplate_OnLoad = function(self)
	self.Text:SetPoint("TOPLEFT",self,"TOPLEFT",_Questic.Padding + _Questic.BorderSize,-_Questic.Padding-_Questic.BorderSize);
	self.Text:SetPoint("TOPRIGHT",self,"TOPRIGHT",-_Questic.Padding - _Questic.BorderSize,-_Questic.Padding-_Questic.BorderSize);
	
	self.BG:SetPoint("BOTTOMLEFT");
	self.BG:SetPoint("BOTTOMRIGHT");
	self.BG:SetPoint("TOP",self.Text,"BOTTOM",0,-_Questic.Padding);
	
	_Questic.SetBackdrop(self.BG,nil,nil,0)
	
	self.EditBox:SetPoint("BOTTOMLEFT",self.BG,"BOTTOMLEFT",_Questic.Padding+_Questic.BorderSize,_Questic.Padding+_Questic.BorderSize);
	self.EditBox:SetPoint("TOPRIGHT",self.BG,"TOPRIGHT",-_Questic.Padding-_Questic.BorderSize,-_Questic.Padding-_Questic.BorderSize);
	
	self.SetValue = function(self,Value,userInput)
		if type(Value) ~= "string" then 
			self.EditBox:ClearFocus(); 
			self.EditBox:SetText(self.Value);
			return false;
		end 
		
		self.EditBox:ClearFocus(); 
		self.EditBox:SetText(Value);
		if type(self.OnValueChanged) == "function" then 
			self:OnValueChanged(Value,self.Value,userInput);
		end 
		self.Value = Value;
	end 
	
	self.SetLines = function(self,Lines)
		Lines = Lines or 1; 
		self:SetHeight(40 + (Lines - 1)*10);
		if Lines > 1 then 
			self.EditBox:SetMultiLine(true);
		else 
			self.EditBox:SetMultiLine(false);
		end 		
	end 
	
	self.Disable = function(self,Disable)
		self.EditBox:EnableMouse(not Disable);	
		if Disable then 
			self.EditBox:SetAlpha(0.5);
			self.Text:SetAlpha(0.5);
		else 
			self.EditBox:SetAlpha(1);					
			self.Text:SetAlpha(1);					
		end 
		self.ItemDisabled = Disable;
	end
	
	
	
	self:SetValue("");
end 

_Questic_ADDON_Globals.Questic_ADDON_OptionsSelectBoxTamplate_OnLoad = function(self)
	self.SelectButton:SetSize(17,17);
	self.SelectButton:SetPoint("BOTTOMRIGHT");
	
	self.SelectButton.Texture:SetPoint("BOTTOMLEFT");
	self.SelectButton.Texture:SetPoint("TOPRIGHT");
	self.SelectButton.Texture:SetTexture("Interface\\BUTTONS\\UI-MicroStream-Yellow");
	
	self.SelectFrame:SetPoint("BOTTOMLEFT");
	self.SelectFrame:SetPoint("TOPRIGHT",self.SelectButton,"TOPLEFT");
	
	self.SelectFrame.Text:SetPoint("BOTTOMLEFT",self.SelectFrame,"BOTTOMLEFT",_Questic.Padding+_Questic.BorderSize,_Questic.Padding+_Questic.BorderSize);
	self.SelectFrame.Text:SetPoint("TOPRIGHT",self.SelectFrame,"TOPRIGHT",-_Questic.Padding-_Questic.BorderSize,-_Questic.Padding-_Questic.BorderSize);

	self.SelectFrame:SetScript("OnMouseDown",function(self)
		if self:GetParent().DropDownItems and #self:GetParent().DropDownItems > 0 then 
			Questic_ADDON_OptionsDropDownMenu:SetOptions(self:GetParent().DropDownItems,"QuesticOptionsDropDownKey",{Point = "TOPLEFT",Ref = self:GetParent(),RefPoint = "BOTTOMLEFT",X = 0,Y = _Questic.BorderSize});
		end 
	end) 
	self.SelectButton:SetScript("OnMouseDown",function(self)
		if self:GetParent().DropDownItems and #self:GetParent().DropDownItems > 0 then 
			Questic_ADDON_OptionsDropDownMenu:SetOptions(self:GetParent().DropDownItems,"QuesticOptionsDropDownKey",{Point = "TOPLEFT",Ref = self:GetParent(),RefPoint = "BOTTOMLEFT",X = 0,Y = _Questic.BorderSize});
		end 
	end)
	
	self.Disable = function(self,Disable)
		self.SelectButton:EnableMouse(not Disable);	
		self.SelectFrame:EnableMouse(not Disable);	
		if Disable then 
			self.Text:SetAlpha(0.5);
			self.SelectFrame.Text:SetAlpha(0.5);
			self.SelectButton:SetAlpha(0.5);
		else 
			self.Text:SetAlpha(1);					
			self.SelectFrame.Text:SetAlpha(1);		
			self.SelectButton:SetAlpha(1);			
		end 
	end

	self.Value = "";
	self.SetValue = function(self,Value,userInput)
		self.SelectFrame.Text:SetText(Value);
		if self.OnValueChanged and self.Value ~= Value then 
			self.OnValueChanged(self,Value,userInput)
		end 
		self.Value = Value;
	end 
	
	self.SetOptions = function(self,Table,Value)
		self.DropDownItems = Table;
		for i,v in ipairs(self.DropDownItems) do 
			local OldFunc = v.Func;
			v.Func = function() 
				if OldFunc then OldFunc() end 
				self:SetValue(v.Text,true);
				Questic_ADDON_OptionsDropDownMenu:SetOptions(nil,"QuesticOptionsDropDownKey",nil,true);
			end
		end 
		self:SetValue(Value);
	end 
	
	if GetLocale() == "ruRU" and self.SelectFrame.Text then 
		local Font,Size,Flag = self.SelectFrame.Text:GetFont();
		self.SelectFrame.Text:SetFont("Fonts\\FRIZQT___CYR.TTF",Size,Flag);
	end 
end

_Questic_ADDON_Globals.Questic_ADDON_ScrollBarTemplate_OnLoad = function(self)
	_Questic.SetBackdrop(self);
	
	self.SetRange = function(self,Value,Displayed,Max)
		self.MaxIndex = Max - Displayed + 1;
		self.MaxValue = Max;
		self.Displayed = Displayed;
		
		self:SetValue(Value);
	end 
	
	self.OnScroll = nil;
	self.Bar:SetWidth(self:GetWidth() - 2*_Questic.BorderSize);
	self.StartIndex = 1;
	self.SetValue = function(self,Value,Increment)
		local PrevValue = self.StartIndex; 
		if Increment then 
			self.StartIndex = self.StartIndex + Value;
		else 
			self.StartIndex = Value;
		end 
		
		if self.StartIndex < 1 then 
			self.StartIndex = 1;
		elseif self.StartIndex > self.MaxIndex then 
			self.StartIndex = self.MaxIndex;
		end 
		
		local ScrollHeight = self:GetHeight() - self.Up:GetHeight() - self.Down:GetHeight();
		self.Bar:SetHeight(self.Displayed/self.MaxValue*ScrollHeight) 
		self.Bar:SetPoint("TOP",0,-(self.Down:GetHeight() + ScrollHeight*(self.StartIndex - 1)/self.MaxValue));
		
		if self.OnScroll then 
			self.OnScroll(self.StartIndex,PrevValue);
		end 
	end 
	
	self:SetScript("OnMouseDown",function(self)
		self:SetScript("OnUpdate",function(self)
			local ScrollHeight = self:GetHeight() - self.Up:GetHeight() - self.Down:GetHeight();
			local _,y = GetCursorPosition();
			local Scale = UIParent:GetEffectiveScale()*self:GetScale();
			local NewPos = self.Up:GetBottom() - y/Scale - self.Bar:GetHeight()/2;
			local NewValue = _Questic.Round(self.MaxValue*NewPos/ScrollHeight) + 1;
			self:SetValue(NewValue);
			if not IsMouseButtonDown("LeftButton") and not IsMouseButtonDown("RightButton") then 
				self:SetScript("OnUpdate",nil);
			end 
		end) 
	end)
	
	self:SetScript("OnMouseUp",function(self)
		self:SetScript("OnUpdate",nil);
	end) 
	
	self:SetScript("OnDragStop",function(self)
		self:SetScript("OnUpdate",nil);
	end) 	
	
	self:SetScript("OnMouseWheel",function(self,delta)
		self:SetValue(-delta,true);
	end)
	
	self.Up:SetPoint("TOP");
	self.Down:SetPoint("BOTTOM");
end
