local _Questic = _Questic_ADDON_Globals;

local SecureButtonPool = CreatePoolCollection();

--local QuestItem,QuestID,Accepted = GetContainerItemQuestInfo(BagID,SlotID);


local RefreshAfterCombat = false; 
local function Refresh(self) 
	if InCombatLockdown() then 
		RefreshAfterCombat = true; 
		return nil 
	end 
	
	RefreshAfterCombat = false; 
	SecureButtonPool:ReleaseAll(); 
	local NumItems = #_Questic.QuestItemList; 
		
	if not _Questic.GetVariable("Enable") or not _Questic.GetVariable("QuestItemBar") or NumItems < 1 then 
		return nil;
	end
	
	local Offset = _Questic.Padding; 
	
	for i,v in ipairs(_Questic.QuestItemList) do 
		if not _Questic.QuestItemBlackList("Check",GetItemInfo(select(7,GetContainerItemInfo(v.BagID,v.SlotID)))) then 
			local texture,itemCount,locked,quality,readable,lootable,itemLink,isFiltered,noValue,itemID = GetContainerItemInfo(v.BagID,v.SlotID);
			self["Item"..i] = SecureButtonPool:Acquire("Questic_ADDON_SecureItemButtonTemplate");
			self["Item"..i].BagID = v.BagID;
			self["Item"..i].SlotID = v.SlotID;
			if itemCount and itemCount ~= 1 then 
				self["Item"..i].Count:SetText(itemCount);
			else 
				self["Item"..i].Count:SetText("");
			end 
			self["Item"..i]:SetAttribute("macrotext","/use "..GetItemInfo(select(7,GetContainerItemInfo(v.BagID,v.SlotID))));
			self["Item"..i].ItemLink = itemLink;
			self["Item"..i].Icon:SetTexture(texture);
			if _Questic.GetVariable("QuestItemBarGrowthDirection") == "Down" then 
				self["Item"..i]:SetPoint("TOP",self,"BOTTOM",0,-Offset);		
			elseif _Questic.GetVariable("QuestItemBarGrowthDirection") == "Up" then 
				self["Item"..i]:SetPoint("BOTTOM",self,"TOP",0,Offset);		
			elseif _Questic.GetVariable("QuestItemBarGrowthDirection") == "Left" then 
				self["Item"..i]:SetPoint("RIGHT",self,"LEFT",-Offset,0);
			else
				self["Item"..i]:SetPoint("LEFT",self,"RIGHT",Offset,0);
			end 
			Offset = Offset + self["Item"..i]:GetWidth() + _Questic.Padding;
			self["Item"..i]:SetScale(_Questic.GetVariable("QuestItemBarScale"));
			self["Item"..i]:Show(); 
			self["Item"..i]:RefreshCD();
		end 
	end 
end 

local IncombatPosUpdate = false; 
local function UpdateParameters(self,ForceReset)
	if InCombatLockdown() then 
		IncombatPosUpdate = true;
		return nil;
	end 
	
	local x,y = _Questic.GetVariable("QuestItemBarPositionx"), _Questic.GetVariable("QuestItemBarPositiony");
	if ForceReset or not x or not y then 
		x,y = UIParent:GetCenter();
		x = (x - self:GetWidth()/2);
		y = (y + self:GetHeight()/2);
	else 
		local PrevScale = self:GetScale();
		x = x*PrevScale;
		y = y*PrevScale;
	end 
	local NewScale = _Questic.GetVariable("QuestItemBarScale");
	for i = 1,#_Questic.QuestItemList do 
		if self["Item"..i] then 
			self["Item"..i]:SetScale(NewScale);
		end 
	end
	self:SetScale(NewScale);
	x = x/NewScale;
	y = y/NewScale;
	self:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",x,y);
	if x ~= _Questic.GetVariable("QuestItemBarPositionx") then 
		_Questic_ADDON_Globals.SaveVariable("QuestItemBarPositionx",x)
	end 
	if _Questic.GetVariable("QuestItemBarPositiony") ~= y then 
		_Questic_ADDON_Globals.SaveVariable("QuestItemBarPositiony",y)
	end 
end 

local function ShowAnchorFrame(self,Show)
	if Show then 
		if #_Questic.QuestItemList < 1 then 
			self.PreviewItem:Show(); 
			self.PreviewItem:ClearAllPoints(); 
			if _Questic.GetVariable("QuestItemBarGrowthDirection") == "Down" then 
				self.PreviewItem:SetPoint("TOP",self,"BOTTOM",0,-_Questic.Padding);		
			elseif _Questic.GetVariable("QuestItemBarGrowthDirection") == "Up" then 
				self.PreviewItem:SetPoint("BOTTOM",self,"TOP",0,_Questic.Padding);		
			elseif _Questic.GetVariable("QuestItemBarGrowthDirection") == "Left" then 
				self.PreviewItem:SetPoint("RIGHT",self,"LEFT",-_Questic.Padding,0);
			else
				self.PreviewItem:SetPoint("LEFT",self,"RIGHT",_Questic.Padding,0);
			end 
		else 
			self.PreviewItem:Hide()
		end 
		self:Show(); 
	else 
		self:Hide(); 
	end 
end 

_Questic_ADDON_Globals.Questic_ADDON_UseQuestItemBar_OnLoad = function(self)
	
	
	SecureButtonPool:CreatePool("BUTTON",UIParent,"Questic_ADDON_SecureItemButtonTemplate");
	
	self.Refresh = Refresh;	
	self.UpdateParameters = UpdateParameters;
	self.Movable = ShowAnchorFrame;
	
	if _Questic.GetVariable("QuestItemBarPositionx") and _Questic.GetVariable("QuestItemBarPositiony") then 
		self:UpdateParameters(); 
	end
	
	self:SetScript("OnEvent",function(self,Event)
		if Event == "PLAYER_REGEN_ENABLED" then 
			self.Text:SetText("Move me!");
			if RefreshAfterCombat then 
				self:Refresh(); 
			end 
			if IncombatPosUpdate then 
				self:UpdateParameters(); 
			end 
		elseif Event == "PLAYER_REGEN_DISABLED" then
			self.Text:SetText("Combat Lock");
		end 
	end) 
	self:RegisterEvent("PLAYER_REGEN_ENABLED");
	self:RegisterEvent("PLAYER_REGEN_DISABLED");
	
	_Questic_ADDON_Globals.SetBackdrop(self);
	
	_Questic_ADDON_Globals.SetBackdrop(self.PreviewItem);
	self.PreviewItem.Texture:SetTexture(136243);
	self.PreviewItem.Texture:SetPoint("BOTTOMLEFT",_Questic.BorderSize,_Questic.BorderSize);
	self.PreviewItem.Texture:SetPoint("TOPRIGHT",-_Questic.BorderSize,-_Questic.BorderSize);
	self.PreviewItem.Texture:SetTexCoord(0.1,0.9,0.1,0.9);
	
	self:Movable();
end 

local function RefreshBagItemCD(self)
	local startTime, duration, isEnabled = GetContainerItemCooldown(self.BagID, self.SlotID);
	self.Cooldown:SetCooldown(startTime, duration);
end 

local function MaskBorderTexture(self,r,g,b,a)
	local Texture = self:CreateTexture(nil,"BACKGROUND");
	Texture:SetAllPoints();
	Texture:SetVertexColor(r,g,b,a);
	Texture:SetMask("Interface\\AddOns\\Questic\\Media\\BorderMaskP5.blp");		
	Texture:SetTexture("Interface\\AddOns\\Questic\\Media\\BasicBackdropTexture.blp");
	return Texture;
end 

_Questic_ADDON_Globals.Questic_ADDON_SecureItemButtonTemplate_OnLoad = function(self)
	self.Icon:SetMask("Interface\\AddOns\\Questic\\Media\\BackdropMaskP5.blp");
	self:RegisterForClicks('RightButtonDown', 'LeftButtonDown');
	
	self.RefreshCD = RefreshBagItemCD; 
	
	self:SetScript("OnEvent",function(self,Event,BagID)
		if BagID then 
			if BagID == self.BagID then 
				self:RefreshCD(); 
			end 
		else 
			self:RefreshCD(); 
		end 
	end) 
	
	self:RegisterEvent("BAG_UPDATE_COOLDOWN");
	--_Questic.SetBackdrop(self);
end 
